package org.xbib.interlibrary.ui;

import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.http.server.InterlibraryServerApplicationModule;
import org.xbib.interlibrary.ui.admin.UserSwitchModel;
import org.xbib.interlibrary.ui.login.LoginModel;
import org.xbib.net.UserProfile;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.application.Application;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.Session;
import org.xbib.settings.Settings;

import java.io.IOException;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

public class UIApplicationModule extends InterlibraryServerApplicationModule {

    private static final Logger logger = Logger.getLogger(UIApplicationModule.class.getName());

    private final HttpUIServer server;

    public UIApplicationModule(HttpUIServer server,
                               Application application,
                               String name,
                               Settings settings) {
        super(application, name, settings);
        this.server = server;
    }

    public Map<String, Object> getLoginForm(String formName,
                                            HttpRequest request,
                                            Session session) {
        TinyMap.Builder<String, Object> builder = LoginModel.getForm(formName, request.getParameter(), session);
        Map<String, Object> map = builder.build();
        String id = (String) map.get("_id");
        if (id == null) {
            logger.log(Level.WARNING, "no ID found for session update, map = " + map);
            return map;
        }
        session.put(id, map);
        return map;
    }

    public Map<String, Object> getUserSwitchForm(String formName,
                                                 HttpRequest request,
                                                 Session session) {
        TinyMap.Builder<String, Object> builder = UserSwitchModel.getForm(formName, request.getParameter(), session);
        builder.put("libraries", server.getConfiguration().getAllLibraryIdentifiers());
        Map<String, Object> map = builder.build();
        String id = (String) map.get("_id");
        if (id == null) {
            logger.log(Level.WARNING, "no ID found for session update, map = " + map);
            return map;
        }
        session.put(id, map);
        return map;
    }

    public void executeUserSwitch(String userSwitchFormName,
                                  HttpRouterContext context) throws IOException {
        HttpRequest request = context.getRequest();
        UserProfile userProfile = context.getAttributes().get(UserProfile.class, "userprofile");
        Session session = context.getAttributes().get(Session.class, "session");
        Map<String, Object> map = getUserSwitchForm(userSwitchFormName, request, session);
        if (map == null) {
            logger.log(Level.WARNING, "no user switch form found");
            return;
        }
        String newEffectiveUser = (String) map.get("library");
        if (newEffectiveUser == null) {
            logger.log(Level.WARNING, "new effective user is null? " + map);
            return;
        }
        userProfile.setEffectiveUserId(newEffectiveUser);
        userProfile.triggerRefresh(true);
        // remove all session attributes as well
        session.clear();
    }

}
