package org.xbib.interlibrary.ui.admin;

import org.xbib.interlibrary.ui.UIApplicationModule;
import org.xbib.interlibrary.ui.statistics.RoyaltiesModel;
import org.xbib.net.ParameterException;
import org.xbib.net.Resource;
import org.xbib.net.UserProfile;
import org.xbib.net.http.HttpResponseStatus;
import org.xbib.net.http.j2html.J2HtmlResource;
import org.xbib.net.http.j2html.J2HtmlResourceHandler;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.resource.HtmlTemplateResourceHandler;
import org.xbib.net.http.server.route.HttpRouterContext;

import java.io.IOException;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;

public class UserSwitchAction extends J2HtmlResourceHandler {

    private static final Logger logger = Logger.getLogger(UserSwitchAction.class.getName());

    public UserSwitchAction() {
    }

    @Override
    protected Resource createResource(HttpRouterContext httpRouterContext)
            throws IOException, ParameterException {
        return new ActionResource(this, httpRouterContext);
    }

    protected static class ActionResource extends J2HtmlResource {

        protected ActionResource(HtmlTemplateResourceHandler templateResourceHandler,
                                 HttpRouterContext httpRouterContext)
                throws IOException, ParameterException {
            super(templateResourceHandler, httpRouterContext);
        }

        @Override
        protected String renderHtml(HttpRouterContext context) {
            logger.log(Level.FINE, "rendering html for user switch action");
            UserProfile userProfile = context.getAttributes().get(UserProfile.class, "userprofile");
            if (userProfile == null || userProfile.getUserId() == null) {
                context.status(HttpResponseStatus.FORBIDDEN).done();
                return "";
            }
            if (!userProfile.getRoles().contains("Admin")) {
                logger.log(Level.WARNING, "trying to user switch from a non-admin user profile " + userProfile + ", forbidden");
                context.status(HttpResponseStatus.FORBIDDEN).done();
                return "";
            }
            HttpRequest request = Objects.requireNonNull(context.getAttributes().get(HttpRequest.class, "request"));
            UIApplicationModule app = Objects.requireNonNull(context.getAttributes().get(UIApplicationModule.class, "app"));
            String id = UserSwitchModel.getFormId("userswitchform", request.getParameter());
            if (id != null) {
                try {
                    app.executeUserSwitch("userswitchform", request.getContext());
                } catch (IOException e) {
                    logger.log(Level.WARNING, e.getMessage(), e);
                    seeOther(contextPath(request, "admin/userswitch/" + id));
                }
                seeOther(contextPath(request, "admin/userswitch/" + id));
            } else {
                logger.log(Level.WARNING, "no userswitch form id found");
                gone();
            }
            return ""; // no content
        }
    }
}
