package org.xbib.interlibrary.ui.admin;

import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.net.Parameter;
import org.xbib.net.ParameterException;
import org.xbib.net.http.server.session.Session;
import org.xbib.net.util.RandomUtil;

import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;

public class UserSwitchModel {

    private static final Logger logger = Logger.getLogger(UserSwitchModel.class.getName());

    public UserSwitchModel() {
    }

    public static String getFormId(String formName, Parameter parameter) {
        try {
            String id = parameter.getAsString(formName, Parameter.Domain.FORM);
            if (id == null || id.isEmpty()) {
                id = parameter.getAsString(formName, Parameter.Domain.PATH);
                if (id == null || id.isEmpty()) {
                    return null;
                }
            }
            return id;
        } catch (ParameterException e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
            return null;
        }
    }


    @SuppressWarnings("unchecked")
    public static TinyMap.Builder<String, Object> getForm(String formName,
                                                          Parameter parameter,
                                                          Session session) {
        TinyMap.Builder<String, Object> builder = TinyMap.builder();
        String formId = getFormId(formName, parameter);
        if (formId == null) {
            logger.log(Level.FINER, "no form ID found for form " + formName + ", using new random ID");
            formId = RandomUtil.randomString(16);
        }
        builder.put("_id", formId);
        builder.put("library", getParameter("library", parameter));
        return builder;
    }

    private static Object getParameter(String key,
                                       Parameter parameter) {
        Object object = null;
        try {
            if (parameter.containsKey(key, Parameter.Domain.PATH)) {
                object = asSingleValueIfPossible(parameter.getAll(key, Parameter.Domain.PATH));
            }
            if (parameter.containsKey(key, Parameter.Domain.FORM)) {
                object = asSingleValueIfPossible(parameter.getAll(key, Parameter.Domain.FORM));
            }
        } catch (Exception e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
        }
        return object;
    }

    /**
     * Unwraps a collection to the first object if the collection is of size 1, otherwise do not unwrap.
     * @param obj if collection of size 1, either the object in the collection, or the collection
     * @return the first object
     */
    private static Object asSingleValueIfPossible(Object obj) {
        if (obj instanceof Collection<?> collection) {
            if (collection.size() == 1) {
                return collection.iterator().next();
            }
        }
        return obj;
    }
}
