package org.xbib.interlibrary.ui.admin;

import org.xbib.interlibrary.http.server.BaseView;
import org.xbib.interlibrary.ui.UIApplicationModule;
import org.xbib.j2html.tags.DomContent;
import org.xbib.j2html.tags.Tag;
import org.xbib.net.ParameterException;
import org.xbib.net.UserProfile;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.application.Application;
import org.xbib.net.http.server.resource.HtmlTemplateResource;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.Session;

import java.util.Collection;
import java.util.Map;
import java.util.Objects;

import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.each;
import static org.xbib.j2html.TagCreator.form;
import static org.xbib.j2html.TagCreator.h1;
import static org.xbib.j2html.TagCreator.input;
import static org.xbib.j2html.TagCreator.main;
import static org.xbib.j2html.TagCreator.option;
import static org.xbib.j2html.TagCreator.select;
import static org.xbib.j2html.TagCreator.span;

public class UserSwitchView extends BaseView {

    public UserSwitchView(HtmlTemplateResource resource,
                          Application application,
                          HttpRequest request) throws ParameterException {
        super(resource, application, request);
    }

    @SuppressWarnings("unchecked")
    @Override
    protected DomContent mainContent(HttpRouterContext context,
                                     Tag<?>... tags) {
        UIApplicationModule app = context.getAttributes().get(UIApplicationModule.class, "app");
        Objects.requireNonNull(app, "app must not be null in context");
        Session session = context.getAttributes().get(Session.class, "session");
        Objects.requireNonNull(session, "session must not be null in context");
        UserProfile userProfile = context.getAttributes().get(UserProfile.class, "userprofile");
        Objects.requireNonNull(userProfile, "user profile must not be null in context");
        Map<String, Object> form = app.getUserSwitchForm("userswitchform", request, session);
        Objects.requireNonNull(form, "form must not be null in session");
        Collection<String> libraries = (Collection<String>) form.get("libraries");
        Objects.requireNonNull(form, "libraries must not be null in form");
        return main().with(div().attr("class", "container").with(
                h1("Bibliothek wechseln"),
                form().attr("class", "form-horizontal mt-2 mb-2")
                        .attr("id", "userswitchform")
                        .attr("method", "post")
                        .attr("action", resource.contextPath(request, "action/userswitch")).with(
                                div().attr("class", "row mb-2 input-group").with(
                                        span().attr("class", "col-sm-3").with(
                                                select().attr("class", "form-select")
                                                        .attr("id", "library")
                                                        .attr("name", "library")
                                                        .with(each(libraries, entry ->
                                                                option().attr("value", entry)
                                                                        .attr(entry.equals(userProfile.getEffectiveUserId()) ? "selected" : "")
                                                                        .withText(entry)
                                                        ))
                                        )
                                ),
                                div().attr("class", "d-grid gap-2 d-md-flex justify-content-md-end").with(
                                        input().attr("type", "hidden")
                                                .attr("name", "userswitchform")
                                                .attr("value", form.get("_id")),
                                        input().attr("class", "btn btn-outline-primary")
                                                .attr("type", "submit")
                                                .attr("id", "userswitchbutton")
                                                .attr("name", "userswitchbutton")
                                                .attr("value", "Bibliothek wählen >>")
                                                .attr("onclick", "javascript:pleaseWait('userswitchbutton', 'userswitchform');return false")
                                )
                        )
        ));
    }
}
