package org.xbib.interlibrary.ui.login;

import org.xbib.interlibrary.ui.UIApplicationModule;
import org.xbib.net.Parameter;
import org.xbib.net.ParameterException;
import org.xbib.net.Resource;
import org.xbib.net.http.j2html.J2HtmlResource;
import org.xbib.net.http.j2html.J2HtmlResourceHandler;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.resource.HtmlTemplateResourceHandler;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.Session;

import java.io.IOException;
import java.util.Map;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;

public class LoginAction extends J2HtmlResourceHandler {

    private static final Logger logger = Logger.getLogger(LoginAction.class.getName());

    public LoginAction() {
    }

    @Override
    protected Resource createResource(HttpRouterContext httpRouterContext) throws IOException, ParameterException {
        return new ActionResource(this, httpRouterContext);
    }

    protected static class ActionResource extends J2HtmlResource {

        protected ActionResource(HtmlTemplateResourceHandler templateResourceHandler,
                                 HttpRouterContext httpRouterContext) throws IOException, ParameterException {
            super(templateResourceHandler, httpRouterContext);
        }

        @Override
        protected String renderHtml(HttpRouterContext context) {
            HttpRequest request = Objects.requireNonNull(context.getAttributes().get(HttpRequest.class, "request"),
                    "request must not be null");
            UIApplicationModule app = Objects.requireNonNull(context.getAttributes().get(UIApplicationModule.class, "app"),
                    "app must not be null");
            Session session = Objects.requireNonNull(context.getAttributes().get(Session.class, "session"),
                    "session must not be null");
            String id = LoginModel.getFormId("loginform", request.getParameter());
            if (id != null) {
                Map<String, Object> map = app.getLoginForm("loginform", request, session);
                session.put(id, map);
                String origin = (String) map.get("_origin");
                try {
                    String originContext = context.getAttributes().get(String.class, "_origin");
                    String originForm = context.getRequest().getParameter().getAsString("_origin", Parameter.Domain.FORM);
                    logger.log(Level.FINE, "origin from context = " + originContext + " origin from parameter = " + originForm);
                } catch (ParameterException e) {
                    logger.log(Level.WARNING, e.getMessage());
                }
                if (context.isAuthenticated()) {
                    seeOther(contextPath(request, origin));
                } else {
                    seeOther(contextPath(request, "login"));
                }
            } else {
                // login form has gone
                gone();
            }
            return ""; // no content for see other
        }
    }
}
