package org.xbib.interlibrary.ui.login;

import org.xbib.interlibrary.http.server.BaseView;
import org.xbib.interlibrary.ui.UIApplicationModule;
import org.xbib.j2html.tags.DomContent;
import org.xbib.j2html.tags.Tag;
import org.xbib.net.Parameter;
import org.xbib.net.ParameterException;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.application.Application;
import org.xbib.net.http.server.resource.HtmlTemplateResource;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.Session;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import static org.xbib.j2html.TagCreator.body;
import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.document;
import static org.xbib.j2html.TagCreator.fieldset;
import static org.xbib.j2html.TagCreator.form;
import static org.xbib.j2html.TagCreator.head;
import static org.xbib.j2html.TagCreator.html;
import static org.xbib.j2html.TagCreator.input;
import static org.xbib.j2html.TagCreator.main;
import static org.xbib.j2html.TagCreator.script;
import static org.xbib.j2html.TagCreator.strong;
import static org.xbib.j2html.TagCreator.title;

public class LoginFormView extends BaseView {

    private static final Logger logger = Logger.getLogger(LoginFormView.class.getName());

    private String formId;

    public LoginFormView(HtmlTemplateResource resource,
                         Application application,
                         HttpRequest request) throws ParameterException {
        super(resource, application, request);
    }

    @Override
    public String createPage(HttpRouterContext context,
                             String pageTitle,
                             Tag<?>... tags) {
        Session session = context.getAttributes().get(Session.class, "session");
        theme = session != null ? (String) session.get("theme") : null;
        return document(
                html().attr("lang", "de").attr("data-bs-theme", theme).with(
                        head(
                                title(pageTitle),
                                headContent()
                        ),
                        body().with(
                                headerContent(),
                                mainContent(context, tags),
                                script().with(themeScript("/action/theme/"))
                        )
                )
        );
    }

    @Override
    protected DomContent mainContent(HttpRouterContext context,
                                     Tag<?>... tags) {
        UIApplicationModule app = context.getAttributes().get(UIApplicationModule.class, "app");
        Session session = context.getAttributes().get(Session.class, "session");
        Map<String, Object> loginForm = app.getLoginForm("loginform", request, session);
        this.formId = (String) loginForm.get("_id");
        // from security domain (form authentication handler)
        String origin = context.getAttributes().get(String.class, "_origin");
        logger.log(Level.FINE, "origin from context attributes = " + origin);
        try {
            String originForm = context.getRequest().getParameter().getAsString("_origin", Parameter.Domain.FORM);
            logger.log(Level.FINE, "origin from parameter = " + originForm);
        } catch (ParameterException e) {
            logger.log(Level.WARNING, e.getMessage());
        }
        return main(loginForm(loginForm, origin));
    }

    @SuppressWarnings("unchecked")
    private DomContent loginForm(Map<String, Object> map, String origin) {
        return div().attr("class", "container-fluid").with(
                div().attr("class", "d-flex justify-content-center").with(
                        div().attr("class", "d-flex flex-column min-vh-100 min-vw-100").with(
                                div().attr("class", "d-flex flex-grow-1 justify-content-center align-items-center").with(
                                div().attr("class", "card")
                                        .attr("style", "width: 32rem;").with(
                                                div().attr("class", "card-header").with(
                                                        strong("Bitte anmelden")
                                                ),
                                                div().attr("class", "card-body").with(
                                                        div().attr("class", "card-text").with(
                                                                form().attr("class", "form")
                                                                .attr("id", "loginform")
                                                                .attr("method", "post")
                                                                .attr("action", resource.contextPath(request, "action/login/" + formId)).with(
                                                                        fieldset().with(
                                                                                div().attr("class", "form-group mb-2").with(
                                                                                        input().attr("class", "form-control border border-secondary")
                                                                                                .attr("type", "text")
                                                                                                .attr("name", "username")
                                                                                                .attr("value", map.get("username"))
                                                                                                .attr("placeholder", "Bitte ISIL eingeben")
                                                                                ),
                                                                                div().attr("class", "form-group mb-2").with(
                                                                                        input().attr("class", "form-control border border-secondary")
                                                                                                .attr("type", "password")
                                                                                                .attr("name", "password")
                                                                                                .attr("value", map.get("password"))
                                                                                                .attr("placeholder", "Bitte Passwort eingeben")
                                                                                ),
                                                                                input().attr("type", "hidden")
                                                                                        .attr("name", "_origin")
                                                                                        .attr("value", origin),
                                                                                input().attr("class", "btn btn-lg btn-success btn-block")
                                                                                        .attr("type", "submit")
                                                                                        .attr("id", "loginbutton")
                                                                                        .attr("name", "loginbutton")
                                                                                        .attr("value", "Anmelden")
                                                                        )
                                                                )
                                                        )
                                                )
                                        )
                                )
                        )
                )
        );
    }
}
