package org.xbib.interlibrary.ui.statistics;

import org.xbib.net.Parameter;
import org.xbib.net.ParameterException;
import org.xbib.net.Resource;
import org.xbib.net.http.HttpResponseStatus;
import org.xbib.net.http.j2html.J2HtmlResource;
import org.xbib.net.http.j2html.J2HtmlResourceHandler;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.resource.HtmlTemplateResourceHandler;
import org.xbib.net.http.server.route.HttpRouterContext;

import java.io.IOException;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;

public class RoyaltiesChangeAction extends J2HtmlResourceHandler {

    private static final Logger logger = Logger.getLogger(RoyaltiesChangeAction.class.getName());

    public RoyaltiesChangeAction() {
    }

    @Override
    protected Resource createResource(HttpRouterContext httpRouterContext)
            throws IOException, ParameterException {
        return new ActionResource(this, httpRouterContext);
    }

    protected static class ActionResource extends J2HtmlResource {

        protected ActionResource(HtmlTemplateResourceHandler templateResourceHandler,
                                 HttpRouterContext httpRouterContext)
                throws IOException, ParameterException {
            super(templateResourceHandler, httpRouterContext);
        }

        @Override
        protected String renderHtml(HttpRouterContext context) {
            HttpRequest request = Objects.requireNonNull(context.getAttributes().get(HttpRequest.class, "request"));
            RoyaltiesModule module = Objects.requireNonNull(context.getAttributes().get(RoyaltiesModule.class, "royalties"));
            String id = RoyaltiesModel.getFormId(request.getParameter());
            if (id != null) {
                try {
                    String number = request.getParameter().getAsString("number", Parameter.Domain.PATH);
                    String value = request.getParameter().getAsString("value", Parameter.Domain.PATH);
                    module.executeRoyaltiesChange(number, value);
                    // reload after change
                    module.executeRoyalties(request.getContext());
                } catch (ParameterException e) {
                    logger.log(Level.WARNING, e.getMessage(), e);
                    context.status(HttpResponseStatus.BAD_REQUEST).done();
                } catch (Exception e) {
                    logger.log(Level.WARNING, e.getMessage(), e);
                    seeOther(contextPath(request, "royalties/" + id));
                }
                seeOther(contextPath(request, "royalties/" + id));
            } else {
                gone();
            }
            return ""; // no content
        }
    }
}
