package org.xbib.interlibrary.ui.statistics;

import org.xbib.net.Parameter;
import org.xbib.net.ParameterException;
import org.xbib.net.Resource;
import org.xbib.net.http.HttpHeaderNames;
import org.xbib.net.http.HttpHeaderValues;
import org.xbib.net.http.HttpResponseStatus;
import org.xbib.net.http.j2html.J2HtmlResource;
import org.xbib.net.http.j2html.J2HtmlResourceHandler;
import org.xbib.net.http.server.resource.HtmlTemplateResourceHandler;
import org.xbib.net.http.server.route.HttpRouterContext;

import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;

public class RoyaltiesClockAction extends J2HtmlResourceHandler {

    private static final Logger logger = Logger.getLogger(RoyaltiesClockAction.class.getName());

    public RoyaltiesClockAction() {
    }

    @Override
    protected Resource createResource(HttpRouterContext httpRouterContext)
            throws IOException, ParameterException {
        return new ActionResource(this, httpRouterContext);
    }

    protected static class ActionResource extends J2HtmlResource {

        protected ActionResource(HtmlTemplateResourceHandler templateResourceHandler,
                                 HttpRouterContext httpRouterContext)
                throws IOException, ParameterException {
            super(templateResourceHandler, httpRouterContext);
        }

        @Override
        protected String renderHtml(HttpRouterContext context) {
            RoyaltiesModule module = Objects.requireNonNull(context.getAttributes().get(RoyaltiesModule.class, "royalties"));
            String id = RoyaltiesModel.getFormId(request.getParameter());
            if (id != null) {
                try {
                    String startDateStr = request.getParameter().getAsString("startdate", Parameter.Domain.PATH);
                    String endDateStr = request.getParameter().getAsString("enddate", Parameter.Domain.PATH);
                    DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ISO_DATE;
                    module.executeRoyaltiesClock(LocalDateTime.parse(startDateStr, dateTimeFormatter), LocalDateTime.parse(endDateStr, dateTimeFormatter));
                    context.status(HttpResponseStatus.OK)
                            .setHeader(HttpHeaderNames.CONTENT_TYPE, HttpHeaderValues.TEXT_PLAIN)
                            .body("OK")
                            .done();
                } catch (ParameterException e) {
                    logger.log(Level.WARNING, e.getMessage(), e);
                    context.status(HttpResponseStatus.BAD_REQUEST).done();
                } catch (Exception e) {
                    logger.log(Level.SEVERE, e.getMessage(), e);
                    context.status(HttpResponseStatus.INTERNAL_SERVER_ERROR)
                            .done();
                }
            }
            return ""; // no content
        }
    }
}
