package org.xbib.interlibrary.ui.statistics;

import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.net.Parameter;
import org.xbib.net.ParameterException;
import org.xbib.net.http.server.session.Session;
import org.xbib.net.util.RandomUtil;

import java.util.Collection;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

public class RoyaltiesModel {

    private static final Logger logger = Logger.getLogger(RoyaltiesModel.class.getName());

    private static final String NAME = "royaltiesform";

    public RoyaltiesModel() {
    }

    public static String getFormId(Parameter parameter) {
        try {
            String id = parameter.getAsString(NAME, Parameter.Domain.FORM);
            if (id == null || id.isEmpty()) {
                id = parameter.getAsString(NAME, Parameter.Domain.PATH);
                if (id == null || id.isEmpty()) {
                    return null;
                }
            }
            return id;
        } catch (ParameterException e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
            return null;
        }
    }

    @SuppressWarnings("unchecked")
    public static TinyMap.Builder<String, Object> getForm(Parameter parameter,
                                                          Session session,
                                                          String defaultHalf) {
        TinyMap.Builder<String, Object> builder = TinyMap.builder();
        String formId = getFormId(parameter);
        if (formId == null) {
            logger.log(Level.FINER, "no form ID found for form " + NAME + ", using new random ID");
            formId = RandomUtil.randomString(16);
        }
        builder.put("_id", formId);
        Map<String, Object> map = (Map<String, Object>) session.get(formId);
        // navigation, e.g. ".../{key}/{value}" -> ".../page/2"
        Object key = getParameter("key", parameter);
        Object value = getParameter("value", parameter);
        Object offset = current("offset", parameter, map, "1");
        Object size = current("size", parameter, map, "10");
        Object page = current("page", parameter, map, "1");
        if ("page".equals(key)) {
            page = value;
            // recompute offset according to page
            long pageLong = Long.parseLong(page.toString());
            long sizeLong = Long.parseLong(size.toString());
            offset = Long.toString(1 + (pageLong - 1) * sizeLong);
        }
        builder.put("offset", offset);
        builder.put("size", size);
        builder.put("page", page);
        Object period = current("period", parameter, map, defaultHalf);
        builder.put("period", period);
        Object numbersearch = current("numbersearch", parameter, map, "");
        builder.put("numbersearch", numbersearch);
        return builder;
    }

    /**
     * Return the most current value given by a key. If the value is not present as a form value, try looking up the path.
     * @param key the key
     * @param parameter the parameter
     * @param map the backing map (persistent in a session)
     * @param defaultValue the default value if there is no other value
     * @return the current value
     */
    private static Object current(String key,
                                  Parameter parameter,
                                  Map<String, Object> map, Object defaultValue) {
        Object defaultObject = map != null ? map.getOrDefault(key, defaultValue) : defaultValue;
        Object object = defaultObject;
        try {
            if (parameter.containsKey(key, Parameter.Domain.PATH)) {
                object = asSingleValueIfPossible(parameter.getAll(key, Parameter.Domain.PATH));
            }
            if (parameter.containsKey(key, Parameter.Domain.FORM)) {
                object = asSingleValueIfPossible(parameter.getAll(key, Parameter.Domain.FORM));
            }
        } catch (Exception e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
        }
        logger.log(Level.FINEST, "current: key = " + key + " value = '" + object + "' " +
                " default = " + defaultObject);
        return object;
    }

    private static Object getParameter(String key,
                                       Parameter parameter) {
        Object object = null;
        try {
            if (parameter.containsKey(key, Parameter.Domain.PATH)) {
                object = asSingleValueIfPossible(parameter.getAll(key, Parameter.Domain.PATH));
            }
        } catch (Exception e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
        }
        return object;
    }


    /**
     * Unwraps a collection to the first object if the collection is of size 1, otherwise do not unwrap.
     * @param obj if collection of size 1, either the object in the collection, or the collection
     * @return the first object
     */
    private static Object asSingleValueIfPossible(Object obj) {
        if (obj instanceof Collection<?> collection) {
            if (collection.size() == 1) {
                return collection.iterator().next();
            }
        }
        return obj;
    }
}
