package org.xbib.interlibrary.ui.statistics;

import org.xbib.net.Parameter;
import org.xbib.net.ParameterException;
import org.xbib.net.Resource;
import org.xbib.net.http.HttpResponseStatus;
import org.xbib.net.http.j2html.J2HtmlResource;
import org.xbib.net.http.j2html.J2HtmlResourceHandler;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.resource.HtmlTemplateResourceHandler;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.Session;

import java.io.IOException;
import java.util.Map;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;

public class RoyaltiesSearchAction extends J2HtmlResourceHandler {

    private static final Logger logger = Logger.getLogger(RoyaltiesSearchAction.class.getName());

    public RoyaltiesSearchAction() {
    }

    @Override
    protected Resource createResource(HttpRouterContext httpRouterContext)
            throws IOException, ParameterException {
        return new ActionResource(this, httpRouterContext);
    }

    protected static class ActionResource extends J2HtmlResource {

        protected ActionResource(HtmlTemplateResourceHandler templateResourceHandler,
                                 HttpRouterContext httpRouterContext)
                throws IOException, ParameterException {
            super(templateResourceHandler, httpRouterContext);
        }

        @SuppressWarnings("unchecked")
        @Override
        protected String renderHtml(HttpRouterContext context) {
            HttpRequest request = Objects.requireNonNull(context.getAttributes().get(HttpRequest.class, "request"));
            String id = RoyaltiesModel.getFormId(request.getParameter());
            if (id != null) {
                try {
                    // keep numbersearch in session and forward back to royalties table
                    String numbersearch = request.getParameter().getAsString("numbersearch", Parameter.Domain.FORM);
                    logger.log(Level.INFO, "received parameter numbersearch = " + numbersearch);
                    Session session = context.getAttributes().get(Session.class, "session");
                    Map<String, Object> map = (Map<String, Object>) session.get(id);
                    map.put("numbersearch", numbersearch);
                    session.put(id, map);
                    seeOther(contextPath(request, "royalties/" + id));
                } catch (ParameterException e) {
                    logger.log(Level.WARNING, e.getMessage(), e);
                    context.status(HttpResponseStatus.BAD_REQUEST).done();
                }
            } else {
                gone();
            }
            return ""; // no content
        }
    }
}
