package org.xbib.interlibrary.ui.statistics;

import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.http.server.BaseView;
import org.xbib.j2html.tags.DomContent;
import org.xbib.j2html.tags.Tag;
import org.xbib.net.ParameterException;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.application.Application;
import org.xbib.net.http.server.resource.HtmlTemplateResource;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.Session;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.Month;
import java.time.Year;
import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import java.util.stream.LongStream;

import static org.xbib.j2html.TagCreator.a;
import static org.xbib.j2html.TagCreator.button;
import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.each;
import static org.xbib.j2html.TagCreator.form;
import static org.xbib.j2html.TagCreator.h1;
import static org.xbib.j2html.TagCreator.h3;
import static org.xbib.j2html.TagCreator.i;
import static org.xbib.j2html.TagCreator.input;
import static org.xbib.j2html.TagCreator.li;
import static org.xbib.j2html.TagCreator.main;
import static org.xbib.j2html.TagCreator.nav;
import static org.xbib.j2html.TagCreator.option;
import static org.xbib.j2html.TagCreator.p;
import static org.xbib.j2html.TagCreator.select;
import static org.xbib.j2html.TagCreator.span;
import static org.xbib.j2html.TagCreator.strong;
import static org.xbib.j2html.TagCreator.table;
import static org.xbib.j2html.TagCreator.tbody;
import static org.xbib.j2html.TagCreator.td;
import static org.xbib.j2html.TagCreator.th;
import static org.xbib.j2html.TagCreator.thead;
import static org.xbib.j2html.TagCreator.tr;
import static org.xbib.j2html.TagCreator.ul;

public class RoyaltiesView extends BaseView {

    private static final Logger logger = Logger.getLogger(RoyaltiesView.class.getName());

    private String formId;

    public RoyaltiesView(HtmlTemplateResource resource,
                         Application application,
                         HttpRequest request) throws ParameterException {
        super(resource, application, request);
    }

    @SuppressWarnings("unchecked")
    @Override
    protected DomContent mainContent(HttpRouterContext context,
                                     Tag<?>... tags) {
        RoyaltiesModule module = context.getAttributes().get(RoyaltiesModule.class, "royalties");
        Session session = context.getAttributes().get(Session.class, "session");
        Map<String, Object> royaltiesForm = module.getRoyaltiesForm(request, session);
        this.formId = (String) royaltiesForm.get("_id");
        Map<String, Object> royaltiesResponse = session.isValid() ?
                (Map<String, Object>) session.get("_statistics_royalties") : Map.of();
        if (royaltiesResponse != null) {
            royaltiesResponse.put("total", Long.valueOf(royaltiesResponse.size()));
        }
        return main().with(
                royaltiesForm(royaltiesForm),
                royaltiesTablePagination(royaltiesForm, royaltiesResponse)
        );
    }

    @SuppressWarnings("unchecked")
    private DomContent royaltiesForm(Map<String, Object> royaltiesForm) {
        List<String> periods = (List<String>) royaltiesForm.get("periods");
        String period = (String) royaltiesForm.get("period");
        return div().attr("class", "container").with(
                h3("VG WORT Meldekontrolle"),
                p("""
Abgeltung der Vergütung gemäß § 60h Abs. 1 UrhG.
Diese Seite dient der Überprüfung der vergütungspflichtigen Aufsatzlieferungen.
Jede Aufsatzlieferung wird über die TAN-Nummer referenziert.
Zur Auswahl stehen die Meldezeiträume gemäß Vereinbarung mit VG WORT.
"""),
                form().attr("class", "form-horizontal mt-2 mb-2")
                        .attr("id", "royaltiesform")
                        .attr("method", "post")
                        .attr("action", resource.contextPath(request, "action/royalties")).with(
                                div().attr("class", "input-group").with(
                                        span().with(
                                                select().attr("class", "form-select")
                                                        .attr("id", "period")
                                                        .attr("name", "period")
                                                        .with(each(periods, entry ->
                                                                option().attr("value", entry)
                                                                        .attr(entry.equals(period) ? "selected" : "")
                                                                        .withText(entry)
                                                        ))
                                        ),
                                        input().attr("type", "hidden")
                                                .attr("name", "royaltiesform")
                                                .attr("value", royaltiesForm.get("_id")),
                                        input().attr("type", "hidden")
                                                .attr("name", "offset")
                                                .attr("value", "1"),
                                        input().attr("type", "hidden")
                                                .attr("name", "page")
                                                .attr("value", "1"),
                                        input().attr("type", "hidden")
                                                .attr("name", "numbersearch")
                                                .attr("value", ""),
                                        input().attr("class", "btn btn-outline-primary")
                                                .attr("type", "submit")
                                                .attr("id", "royaltiesbutton")
                                                .attr("name", "royaltiesbutton")
                                                .attr("value", "Anzeigen >>")
                                                .attr("onclick", "javascript:pleaseWait('royaltiesbutton', 'royaltiesform')")
                                )
                        )
        );
    }

    @SuppressWarnings("unchecked")
    private DomContent royaltiesTablePagination(Map<String, Object> form,
                                                Map<String, Object> royaltiesResponse) {
        if (royaltiesResponse == null) {
            return emptyResult();
        }
        String isil = (String) form.get("isil");
        // paging control
        int limit = 1000000; // disable
        int offset = Integer.parseInt((String) form.get("offset"));
        if (offset == 0) {
            offset = 1;
        }
        if (offset > limit) {
            offset = limit;
        }
        int size = Integer.parseInt((String) form.get("size"));
        if (size == 0) {
            size = 1;
        }
        if (size > limit) {
            size = limit;
        }
        long page = form.containsKey("page") ? Long.parseLong((String) form.get("page")) : 0L;
        long currentpage = page != 0L ? page : (offset / size);
        if (currentpage == 0L) {
            currentpage = 1L;
        }
        if (currentpage > limit) {
            currentpage = limit;
        }
        long total = royaltiesResponse.containsKey("total") ? (Long) royaltiesResponse.get("total") : 0L;
        int pagelimit = size;
        int pages = 5;
        long lastpage = total != 0 ? total / pagelimit : 0L;
        if (total != 0L && (total % pagelimit != 0L)) {
            lastpage++;
        }
        long first = Math.max(1, currentpage - 2);
        long last = Math.min(first + pages, lastpage);

        // page info complete
        logger.log(Level.INFO, "offset = " + offset + " size = " + size + " page = " + page + " total = " + total);

        LocalDateTime min = royaltiesResponse.values().stream()
                .filter(v -> v instanceof Map)
                .map(v -> (String) ((Map<String, Object>)v).get("Datum"))
                .filter(Objects::nonNull)
                .map(LocalDateTime::parse)
                .min(LocalDateTime::compareTo)
                .orElse(null);

        LocalDateTime max = royaltiesResponse.values().stream()
                .filter(v -> v instanceof Map)
                .map(v ->  (String) ((Map<String, Object>)v).get("Datum"))
                .filter(Objects::nonNull)
                .map(LocalDateTime::parse)
                .max(LocalDateTime::compareTo)
                .orElse(null);

        String period = (String) form.get("period");
        LocalDate finalReportDate = LocalDate.now().plusMonths(1L);
        if (period != null) { // always != null
            Year year = Year.of(Integer.parseInt(period.substring(0, 4)));
            Month month = Month.of((Integer.parseInt(period.substring(5, 6)) - 1) * 6 + 1);
            if (month.getValue() > 6) {
                year = year.plusYears(1L);
                month = Month.JANUARY;
            } else {
                month = Month.JULY;
            }
            YearMonth yearMonth = YearMonth.of(year.getValue(), month.getValue());
            finalReportDate = addWeekDays(LocalDate.of(year.getValue(), month.getValue(), 1), 20);
        }

        DateTimeFormatter formatter = DateTimeFormatter.ofLocalizedDate(FormatStyle.FULL);

        Map<Object, Long> m = royaltiesResponse.values().stream()
                .filter(v -> v instanceof Map)
                .map(v -> ((Map<String, Object>)v).get("Modus"))
                .filter(Objects::nonNull)
                .collect(Collectors.groupingBy(Function.identity(), Collectors.counting()));

        long totalState = m.getOrDefault("P", 0L);
        long totalDirect = m.getOrDefault("E", 0L);

        String numbersearch = (String) form.get("numbersearch");

        List<Map<String, Object>> rows = royaltiesResponse.values().stream()
                .filter(v -> v instanceof Map)
                .filter(v -> filterRow((Map<String, Object>)v, numbersearch))
                .skip(offset - 1)
                .limit(size)
                .map(v -> filterColumns((Map<String, Object>)v, isil != null))
                .toList();

        boolean isEmptyTable = rows.isEmpty();

        return isEmptyTable ? null : div().with(
                div().attr("class","container").with(
                        p().with(form().attr("class", "d-flex col-sm-3")
                                .attr("role", "search")
                                .attr("method", "post")
                                .attr("action", resource.contextPath(request, "action/royalties/search/" + formId))
                                .with(input().attr("class", "form-control me-2")
                                                .attr("type", "search")
                                                .attr("placeholder", "Suche nach Nummer")
                                                .attr("aria-label", "Suche nach Nummer")
                                                .attr("name", "numbersearch")
                                                .attr("value", numbersearch),
                                        button().attr("class", "btn btn-outline-primary")
                                                .attr("type", "submit")
                                                .with(i().attr("class", "bi bi-search")))),
                        p().withText("Angezeigt werden vergütungspflichtige Aufsatzlieferungen aus dem Zeitraum " +
                                (min != null ? min.format(formatter) : "[unbekannt]") +
                                " bis " +
                                (max != null ? max.format(formatter) : "[unbekannt]") +
                                " (einschliesslich)"),
                        div().attr("class", "row").with(
                                div().attr("class", "col").with(strong(h1("" + totalDirect)), span(" Anzahl der Direktlieferungen mit Vergütung durch bestellende Bibliothek an VG WORT, " +
                                        "siehe Eintrag \"Direkt\" in Spalte \"Vergütung\" bzw. in den Download-Dateien: Eintrag \"e\" in Spalte \"delivery\"")),
                                div().attr("class", "col").with(strong(h1("" + totalState)), span(" Anzahl der Lieferungen mit Vergütung durch Bund und Länder über KMK an VG WORT, " +
                                        "siehe Eintrag \"KMK\" in Spalte \"Vergütung\" bzw. in den Download-Dateien: Eintrag \"p\" in Spalte \"delivery\""))
                        ),
                        p().attr("class", "alert alert-warning").with(span("Rückmeldungen und Beanstandungen können bis spätestens " + finalReportDate.format(formatter) +
                                        " an das Team Online-Fernleihe im hbz gemeldet werden: "),
                                a().attr("href", "mailto:fernleihe@hbz-nrw.de").withText("fernleihe@hbz-nrw.de")),
                        //p().withText("Betätigungen des Buttons \"ändern\" in Spalte \"Aktion\" werden protokolliert."),
                        div().attr("class", "btn-toolbar").with(
                                div().attr("class", "btn-group m-3").with(
                                    form().attr("method", "post")
                                            .attr("action", resource.contextPath(request, "action/royalties/download/xml"))
                                            .with(
                                                input().attr("type", "hidden")
                                                        .attr("name", "royaltiesform")
                                                        .attr("value", form.get("_id")),
                                                button().attr("type", "submit")
                                                        .attr("class", "btn btn-lg btn-success")
                                                        .with(i().attr("class", "bi-filetype-xml"),
                                                                span().withText(" Download XML")))),
                                div().attr("class", "btn-group m-3").with(
                                    form().attr("method", "post")
                                            .attr("action", resource.contextPath(request, "action/royalties/download/csv"))
                                            .with(
                                                input().attr("type", "hidden")
                                                        .attr("name", "royaltiesform")
                                                        .attr("value", form.get("_id")),
                                                button().attr("type", "submit")
                                                        .attr("class", "btn btn-lg btn-success")
                                                        .with(i().attr("class", "bi-filetype-csv"),
                                                                span().withText(" Download CSV")))),
                                div().attr("class", "btn-group m-3").with(
                                    form().attr("method", "post")
                                            .attr("action", resource.contextPath(request, "action/royalties/download/xlsx"))
                                            .with(
                                                input().attr("type", "hidden")
                                                        .attr("name", "royaltiesform")
                                                        .attr("value", form.get("_id")),
                                                button().attr("type", "submit")
                                                        .attr("class", "btn btn-lg btn-success")
                                                        .with(i().attr("class", "bi-filetype-xlsx"),
                                                                span().withText(" Download XLSX (Excel)"))))
                        )
                ),
                pagination("action/royalties/", page, first, last, lastpage),
                div().attr("class","container").with(
                        div().withText("Seite " + page + " von " + lastpage + " mit insgesamt " + total + " Einträgen"),
                        table().attr("class", "table table-sm table-bordered")
                            .attr("style", "width: 100%").with(
                            thead().with(
                                    tr().with(each(rows.getFirst().keySet(), cell ->
                                            th().attr("scope", "col").withText(cell))
                                            //th().attr("scope", "col").withText("Aktion"))
                            ),
                            tbody().with(each(rows, row ->
                                    tr().with(each(row.values(), cell ->
                                            td().withText(cell != null ? cell.toString() : ""))
                                            /*td().with(form()
                                                    .attr("method", "post")
                                                    .attr("action", resource.contextPath(request, "action/royalties/change/" + row.get("Nummer") + "/" +
                                                                    invertTranslatedMode((String) row.get("Vergütung"))))
                                                    .with(
                                                        input().attr("type", "hidden")
                                                                .attr("name", "royaltiesform")
                                                                .attr("value", form.get("_id")),
                                                        input().attr("type", "submit")
                                                                .attr("class", "btn btn-xs btn-outline-" + inverseTheme)
                                                                .attr("value", "ändern")
                                            )*/
                                    )))
                            )
                    )
                )
        );
    }

    private static DomContent emptyResult() {
        return div().attr("class", "container").with(
                p("Keine anzuzeigenden Ergebnisse. Bitte Vergütungszeitraum auswählen.")
        );
    }

    private static boolean filterRow(Map<String, Object> map, String numbersearch) {
        return numbersearch == null || numbersearch.isEmpty() || numbersearch.equals(map.get("Nummer"));
    }

    private Map<String, Object> filterColumns(Map<String, Object> map, boolean withIsil) {
        TinyMap.Builder<String, Object> builder = TinyMap.builder();
        builder.putAll(map);
        if (withIsil) {
            builder.remove("ISIL");
        } else {
            builder.put("ISIL", ((String) builder.get("ISIL")).replace('-', '\u2011'));
        }
        builder.remove("Datum");
        String mode = (String) builder.get("Modus");
        builder.remove("Modus");
        builder.put("Vergütung", translateMode(mode));
        return builder.build();
    }

    private static String translateMode(String mode) {
        if (mode == null) {
            return "?";
        }
        return switch (mode) {
            case "P" -> "KMK";
            case "E" -> "Direkt";
            default -> mode;
        };
    }

    private static String invertTranslatedMode(String mode) {
        if (mode == null) {
            return "?";
        }
        return switch (mode) {
            case "KMK" -> "E";
            case "Direkt" -> "P";
            default -> mode;
        };
    }

    private static LocalDate addWeekDays(LocalDate localDate, int weekDays) {
        int i = weekDays;
        if (i <= 1) {
            return localDate;
        }
        while (i > 1 ) {
            localDate = localDate.plusDays(1);
            switch (localDate.getDayOfWeek()) {
                case MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY -> i--;
            }
        }
        return localDate;
    }

    private DomContent pagination(String action, long page, long first, long last, long lastpage) {
        if (first >= last) {
            return span();
        }
        long prev = Math.max(1L, page - 1L);
        long next = Math.min(page + 1L, last);
        List<Long> pages = LongStream.range(first, last + 1).boxed().toList();
        return nav().attr("class", "mt-2").attr("aria-label", "Navigation").with(
            ul().attr("class", "pagination justify-content-center").with(
                    li().attr("class", page == 1L ? "page-item disabled" : "page-item").with(
                            a().attr("class", "page-link")
                                    .attr("href", resource.contextPath(request, action + formId + "/page/1")).with(
                                            i().attr("class", "bi-caret-left-square")
                                    )
                    ),
                    li().attr("class", page == prev ? "page-item disabled" : "page-item").with(
                            a().attr("class", "page-link")
                                    .attr("href", resource.contextPath(request, action + formId + "/page/" + prev)).with(
                                            i().attr("class", "bi-caret-left")
                                    )
                    ),
                    each(pages, p ->
                            li().attr("class", page == p ? "page-item disabled" : "page-item").with(
                                    a().attr("class", "page-link")
                                            .attr("href", resource.contextPath(request, action + formId + "/page/" + p)).withText(String.valueOf(p))
                            )
                    ),
                    li().attr("class", page == next ? "page-item disabled" : "page-item").with(
                            a().attr("class", "page-link")
                                    .attr("href", resource.contextPath(request, action + formId + "/page/" + next)).with(
                                            i().attr("class", "bi-caret-right")
                                    )
                    ),
                    li().attr("class", page == lastpage ? "page-item disabled" : "page-item").with(
                            a().attr("class", "page-link")
                                    .attr("href", resource.contextPath(request, action + formId + "/page/" + lastpage)).with(
                                            i().attr("class", "bi-caret-right-square")
                                    )
                    )
            )
        );
    }
}
