package org.xbib.interlibrary.ui.theme;

import org.xbib.net.Parameter;
import org.xbib.net.ParameterException;
import org.xbib.net.UserProfile;
import org.xbib.net.http.HttpHeaderNames;
import org.xbib.net.http.HttpHeaderValues;
import org.xbib.net.http.HttpResponseStatus;
import org.xbib.net.http.server.HttpHandler;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.Session;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ThemeController implements HttpHandler {

    private static final Logger logger = Logger.getLogger(ThemeController.class.getName());

    public ThemeController() {
    }

    @Override
    public void handle(HttpRouterContext httpRouterContext) throws IOException {
        try {
            Parameter parameter = httpRouterContext.getRequest().getParameter();
            String theme = parameter.getAsString("theme", Parameter.Domain.PATH);
            if ("light".equals(theme) || "dark".equals(theme) || "auto".equals(theme)) {
                logger.log(Level.FINE, "setting theme to " + theme);
                Session session = httpRouterContext.getAttributes().get(Session.class, "session");
                if (session != null) {
                    session.put("theme", theme);
                }
                // also in user profile for spanning sessions
                UserProfile userProfile = httpRouterContext.getAttributes().get(UserProfile.class, "userprofile");
                if (userProfile != null) {
                    userProfile.getAttributes().put("theme", theme);
                }
                httpRouterContext.status(HttpResponseStatus.OK)
                        .setHeader(HttpHeaderNames.CONTENT_TYPE, HttpHeaderValues.APPLICATION_JSON)
                        .body("{}")
                        .done();
            } else {
                httpRouterContext.status(HttpResponseStatus.BAD_REQUEST).done();
            }
        } catch (ParameterException e) {
            httpRouterContext.status(HttpResponseStatus.BAD_REQUEST)
                    .done();
        }
    }
}
